"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.plugin = void 0;
/**
 * Tencent is pleased to support the open source community by making CloudBaseFramework - 云原生一体化部署工具 available.
 *
 * Copyright (C) 2020 THL A29 Limited, a Tencent company.  All rights reserved.
 *
 * Please refer to license text included with this package for license details.
 */
const framework_core_1 = require("@cloudbase/framework-core");
const framework_plugin_container_1 = require("@cloudbase/framework-plugin-container");
const builder_1 = require("./builder");
function resolveInputs(inputs, defaultInputs) {
    return Object.assign({}, defaultInputs, inputs);
}
class DenoPlugin extends framework_core_1.Plugin {
    constructor(name, api, inputs) {
        super(name, api, inputs);
        this.name = name;
        this.api = api;
        this.inputs = inputs;
        const DEFAULT_INPUTS = {
            dockerImage: 'debian:buster-slim',
            // runtime example: v1.3.0
            runtime: 'latest',
            // denonVersion example: @2.4.0
            denonVersion: '',
            entry: '',
            autoBuild: true,
            serviceName: 'deno-app',
            servicePath: '/deno-app',
            projectPath: '.',
        };
        this.resolvedInputs = resolveInputs(this.inputs, DEFAULT_INPUTS);
        this.denoBuilder = new builder_1.DenoBuilder({
            projectPath: this.api.projectPath,
        });
    }
    /**
     * 初始化资源
     */
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug('DenoPlugin: init', this.resolvedInputs);
        });
    }
    /**
     * 生成功能代码
     */
    genCode() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    /**
     * 构建资源
     */
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug('DenoPlugin: build', this.resolvedInputs);
            const { projectPath, dockerImage, runtime, denonVersion, entry, autoBuild, serviceName, servicePath, } = this.resolvedInputs;
            // 构建 deno 中间产物
            this.buildOutput = yield this.denoBuilder.build(projectPath || '.', {
                dockerImage,
                runtime,
                denonVersion,
                entry,
                autoBuild,
                name: serviceName,
                path: servicePath,
            });
            const container = this.buildOutput.containers[0];
            this.containerPlugin = new framework_plugin_container_1.plugin('container', this.api, resolveInputs({ localAbsolutePath: container.source }, this.resolvedInputs));
            yield this.containerPlugin.init();
            // 构建 container 最终产物
            yield this.containerPlugin.build();
        });
    }
    /**
     * 部署资源
     */
    deploy() {
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug('DenoPlugin: deploy', this.resolvedInputs, this.buildOutput);
            yield this.containerPlugin.deploy();
            yield this.denoBuilder.clean();
            this.api.logger.info(`${this.api.emoji('🚀')} Deno 应用部署成功`);
        });
    }
    /**
     * 将资源编译成 SAM 描述
     */
    compile() {
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug('DenoPlugin: compile', this.resolvedInputs);
            return this.containerPlugin.compile();
        });
    }
    /**
     * 移除资源
     */
    remove() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    /**
     * 执行自定义命令
     */
    run() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
}
exports.plugin = DenoPlugin;
